import copy

from apispec import BasePlugin, yaml_utils
from apispec.exceptions import APISpecError
import falcon


class FalconPlugin(BasePlugin):
    """APISpec plugin for Falcon"""

    def __init__(self, app):
        super(FalconPlugin, self).__init__()
        self._app = app

    @staticmethod
    def _generate_resource_uri_mapping(app):
        routes_to_check = copy.copy(app._router._roots)

        mapping = {}
        for route in routes_to_check:
            uri = route.uri_template
            resource = route.resource
            mapping[resource] = uri
            routes_to_check.extend(route.children)

        return mapping

    def path_helper(self, operations, resource, **kwargs):
        """Path helper that allows passing a Falcon resource instance."""
        resource_uri_mapping = self._generate_resource_uri_mapping(self._app)

        if resource not in resource_uri_mapping:
            raise APISpecError("Could not find endpoint for resource {0}".format(resource))

        operations.update(yaml_utils.load_operations_from_docstring(resource.__doc__) or {})
        path = resource_uri_mapping[resource]

        for method in falcon.constants.HTTP_METHODS:
            http_verb = method.lower()
            method_name = "on_" + http_verb
            if getattr(resource, method_name, None) is not None:
                method = getattr(resource, method_name)
                docstring_yaml = yaml_utils.load_yaml_from_docstring(method.__doc__)
                operations[http_verb] = docstring_yaml or dict()

        return path



import codecs
import os
import re
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


with open("README.md") as f:
    long_description = f.read()

setup(
    name="falcon-apispec",
    version=find_version("falcon_apispec", "version.py"),

    author="Aly Sivji",
    author_email="alysivji@gmail.com",

    description="Falcon plugin for apispec documentation generator.",
    long_description=long_description,
    long_description_content_type="text/markdown",

    url="https://github.com/alysivji/falcon-apispec",
    project_urls={
        "Funding": "https://falconframework.org/#sectionSupportFalconDevelopment",
        "Source": "https://github.com/alysivji/falcon-apispec",
        "Tracker": "https://github.com/alysivji/falcon-apispec/issues",
    },

    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    keywords="falcon apispec swagger openapi specification documentation spec rest api",

    install_requires=[
        "PyYAML>=3.10",
        "apispec>=1.0",
        "falcon",
    ],
    packages=find_packages(exclude=["tests", ]),
    test_suite='tests',

    zip_safe=False,
)
